@@ -7,6 +7,7 @@ from social_interaction_cloud.abstract_connector import AbstractSICConnector
 
 
 class RobotPosture(Enum):
+    """Enumeration of available postures on the Nao and Pepper robot."""
     STAND = 'Stand'
     STANDINIT = 'StandInit'
     STANDZERO = 'StandZero'
@@ -20,6 +21,7 @@ class RobotPosture(Enum):
 
 
 class VisionType(Enum):
+    """Enumeration of the available vision events."""
     FACE = 'onFaceRecognized'
     PEOPLE = 'onPersonDetected'
     EMOTION = 'onEmotionDetected'
@@ -29,6 +31,7 @@ class VisionType(Enum):
     TRACKING = 'onObjectTracked'
 
     def as_service(self):
+        """Returns the service that generates the vision events."""
         if self == VisionType.FACE:
             return 'face_recognition'
         elif self == VisionType.PEOPLE:
@@ -46,23 +49,24 @@ class VisionType(Enum):
 
 
 class NoCallbackException(Exception):
+    """Exception raised when a required callback function is not provided"""
     pass
 
 
 class Action:
     """
-    Encapsulation class for BasicSICConnector method calls.
+    Encapsulation class for callables.
 
-    The BasicSICConnector method call is executed when the perform() method is called. To create a waiting action,
+    The callable is executed when the perform() method is called. For synchronous calls,
     a threading.Event() object should be provided as lock.
     """
 
     def __init__(self, action: callable, *args, lock: Event = None):
         """
 
-        :param action: a callable from the BasicSICConnector
+        :param action: a callable.
         :param args: optional input arguments for the callable
-        :param lock: optional lock to create a waiting Action.
+        :param lock: optional lock to force synchronicity.
         """
         self.action = action
         self.args = args
@@ -79,11 +83,14 @@ class Action:
 
 class BasicSICConnector(AbstractSICConnector):
     """
-    Basic implementation of AbstractSICConnector. It serves a connector to the Social Interaction Cloud.
+    Basic implementation of AbstractSICConnector.
+
+    It serves a connector to the Social Interaction Cloud.
     The base mechanism is that a callback function can be registered for each robot action. When the action returns a
     result (e.g. a ActionDone event) the callback is called once and removed. Only for touch and vision events a
     persistent callback can be registered.
 
+    Each action can be run synchronously (main thread waits for result) and asynchronously (main thread continues).
     """
 
     def __init__(self, server_ip: str, dialogflow_language: str = None,
@@ -94,9 +101,9 @@ class BasicSICConnector(AbstractSICConnector):
         :param dialogflow_language: the full language key to use in Dialogflow (e.g. en-US)
         :param dialogflow_key_file: path to Google's Dialogflow key file (JSON)
         :param dialogflow_agent_id: ID number of Dialogflow agent to be used (project ID)
-        :param tts_language: the full language key to use in TTS (e.g. en-US)
         :param tts_key_file: path to Google's TTS key file (JSON)
         :param tts_voice: ID number of TTS voice to be used
+        :param sentiment: use of sentiment analysis
         :param stereo_camera: use of stereo_camera
         """
         self.__action_listeners = {}
@@ -226,7 +233,6 @@ class BasicSICConnector(AbstractSICConnector):
         :param max_duration: maximum time to listen in seconds (int)
         :param callback: callback function that will be called when a result (or fail) becomes available
         :param sync: lets main thread wait on this action
-        :return:
         """
         inner_callback = callback
         lock = None
@@ -252,7 +258,6 @@ class BasicSICConnector(AbstractSICConnector):
         :param duration: number of second of audio that will be recorded.
         :param callback: callback function that will be called when the audio is recorded.
         :param sync: lets main thread wait on this action
-        :return:
         """
         inner_callback = callback
         lock = None
@@ -303,23 +308,29 @@ class BasicSICConnector(AbstractSICConnector):
     # Vision                  #
     ###########################
 
-    def take_picture(self, callback: callable = None, sync: bool = True, chain: bool = False) -> None:
+    def take_picture(self, callback: callable = None, sync: bool = True, load: bool = False) -> None:
         """
         Take a picture. Location of the stored picture is returned via callback.
 
         :param callback:
         :param sync:
-        :param chain:
+        :param load:
         :return:
         """
         self.__process_action(super(BasicSICConnector, self).take_picture,
-                              callback=callback, event='onNewPictureFile', sync=sync, chain=chain)
+                              callback=callback, event='onNewPictureFile', sync=sync, load=load)
 
     def subscribe_vision_listener(self, vision_type: VisionType, callback: callable = None, continuous: bool = False,
-                                  sync: bool = True, chain: bool = False) -> None:
-        # not continuous stops after one hit, continuous continues
-        # synch wait until (first) hit
-        # chain wait until any or all (first) hit
+                                  sync: bool = True, load: bool = False) -> None:
+        """
+        Subscribe a
+        :param vision_type:
+        :param callback:
+        :param continuous:
+        :param sync:
+        :param load:
+        """
+
         inner_callback = callback
         lock = None
 
@@ -332,7 +343,7 @@ class BasicSICConnector(AbstractSICConnector):
         if not inner_callback:
             raise NoCallbackException('A vision listener requires a callback')
 
-        if chain:
+        if load:
             self.enable_service(vision_type.as_service())
             sleep(1)  # give the service some time to load
             self.__loaded_actions.append(Action(self.__start_vision_recognition, str(vision_type.value), inner_callback,
@@ -345,16 +356,13 @@ class BasicSICConnector(AbstractSICConnector):
                 lock.wait()
 
     def unsubscribe_vision_listener(self, vision_type: VisionType):
+        """
+
+        :param vision_type:
+        """
         self.__stop_vision_recognition(str(vision_type.value))
 
     def __build_vision_stopping_callback(self, vision_type: VisionType, original_callback: callable = None):
-        """
-        Builds callback function for a single use vision listener.
-
-        :param vision_type:
-        :param original_callback:
-        :return:
-        """
         def callback(*args):
             if original_callback:
                 original_callback(*args)
@@ -379,17 +387,20 @@ class BasicSICConnector(AbstractSICConnector):
     ###########################
 
     def subscribe_event_listener(self, event: str, callback: callable, continuous: bool = False,
-                                 sync: bool = True, chain: bool = False) -> None:
-        """
-        Subscribe an event listener. The callback function will be called each time the event becomes available.
+                                 sync: bool = True, load: bool = False) -> None:
+        """Subscribe an event listener.
 
-        :param event:
-        :param callback:
-        :return:
+        :param event: any event.
+        :param callback: The callback function to be called when the event becomes available.
+        :param continuous: when true the callback will be triggered every time, when false the callback will be
+        triggered only once.
+        :param sync: lets main thread wait on this action
+        :param load: does not subscribe the event listeners, but loads it instead.
+        See run_loaded_actions() for subscribing loaded listeners.
         """
         # not continuous stops after one hit, continuous continues
         # synch wait until (first) hit
-        # chain wait until any or all (first) hit
+        # load wait until any or all (first) hit
         inner_callback = callback
         lock = None
 
@@ -402,7 +413,7 @@ class BasicSICConnector(AbstractSICConnector):
         if not inner_callback:
             raise NoCallbackException('An event listener requires a callback')
 
-        if chain:
+        if load:
             self.__loaded_actions.append(Action(self.__register_event_listener, event, inner_callback, lock=lock))
         else:
             self.__register_event_listener(event, inner_callback)
@@ -411,10 +422,9 @@ class BasicSICConnector(AbstractSICConnector):
 
     def unsubscribe_event_listener(self, event: str) -> None:
         """
-        Unsubscribe touch listener.
+        Unsubscribe event listener.
 
         :param event:
-        :return:
         """
         self.__unregister_event_listener(event)
 
@@ -430,113 +440,109 @@ class BasicSICConnector(AbstractSICConnector):
     # Robot actions           #
     ###########################
 
-    def say_text_to_speech(self, text: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def say_text_to_speech(self, text: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).say_text_to_speech, text,
-                              callback=callback, event='PlayAudioDone', sync=sync, chain=chain)
+                              callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
-    def set_language(self, language_key: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_language(self, language_key: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_language, language_key,
-                              callback=callback, event='LanguageChanged', sync=sync, chain=chain)
+                              callback=callback, event='LanguageChanged', sync=sync, load=load)
 
-    def set_idle(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_idle(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_idle,
-                              callback=callback, event='SetIdle', sync=sync, chain=chain)
+                              callback=callback, event='SetIdle', sync=sync, load=load)
 
-    def set_non_idle(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_non_idle(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_non_idle,
-                              callback=callback, event='SetNonIdle', sync=sync, chain=chain)
+                              callback=callback, event='SetNonIdle', sync=sync, load=load)
 
-    def start_looking(self, seconds: int, channels: int = 1, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def start_looking(self, seconds: int, channels: int = 1, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_looking, seconds, channels,
-                              callback=callback, event='WatchingStarted', sync=sync, chain=chain)
+                              callback=callback, event='WatchingStarted', sync=sync, load=load)
 
-    def stop_looking(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def stop_looking(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).stop_looking,
-                              callback=callback, event='WatchingDone', sync=sync, chain=chain)
+                              callback=callback, event='WatchingDone', sync=sync, load=load)
 
-    def say(self, text: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def say(self, text: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).say, text,
-                              callback=callback, event='TextDone', sync=sync, chain=chain)
+                              callback=callback, event='TextDone', sync=sync, load=load)
 
-    def say_animated(self, text: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def say_animated(self, text: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).say_animated, text,
-                              callback=callback, event='TextDone', sync=sync, chain=chain)
+                              callback=callback, event='TextDone', sync=sync, load=load)
 
-    def stop_talking(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def stop_talking(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).stop_talking,
-                              callback=callback, event='TextDone', sync=sync, chain=chain)
+                              callback=callback, event='TextDone', sync=sync, load=load)
 
-    def do_gesture(self, gesture: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def do_gesture(self, gesture: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).do_gesture, gesture,
-                              callback=callback, event='GestureDone', sync=sync, chain=chain)
+                              callback=callback, event='GestureDone', sync=sync, load=load)
 
-    def load_audio(self, audio_file: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def load_audio(self, audio_file: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).load_audio, audio_file,
-                              callback=callback, event='LoadAudioDone', sync=sync, chain=chain)
+                              callback=callback, event='LoadAudioDone', sync=sync, load=load)
 
-    def play_audio(self, audio_file: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def play_audio(self, audio_file: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).play_audio, audio_file,
-                              callback=callback, event='PlayAudioDone', sync=sync, chain=chain)
+                              callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
-    def play_loaded_audio(self, audio_identifier: int, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def play_loaded_audio(self, audio_identifier: int, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).play_loaded_audio, audio_identifier,
-                              callback=callback, event='PlayAudioDone', sync=sync, chain=chain)
+                              callback=callback, event='PlayAudioDone', sync=sync, load=load)
 
-    def clear_loaded_audio(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def clear_loaded_audio(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).clear_loaded_audio,
-                              callback=callback, event='ClearLoadedAudioDone', sync=sync, chain=chain)
+                              callback=callback, event='ClearLoadedAudioDone', sync=sync, load=load)
 
-    def set_eye_color(self, color: str, callback: callable = None, sync: bool = True, chain: bool = False) -> None:
+    def set_eye_color(self, color: str, callback: callable = None, sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_eye_color, color,
-                              callback=callback, event='EyeColourDone', sync=sync, chain=chain)
+                              callback=callback, event='EyeColourDone', sync=sync, load=load)
 
-    def set_ear_color(self, color: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_ear_color(self, color: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_ear_color, color,
-                              callback=callback, event='EarColourDone', sync=sync, chain=chain)
+                              callback=callback, event='EarColourDone', sync=sync, load=load)
 
-    def set_head_color(self, color: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_head_color(self, color: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_head_color, color,
-                              callback=callback, event='HeadColourDone', sync=sync, chain=chain)
+                              callback=callback, event='HeadColourDone', sync=sync, load=load)
 
-    def set_led_color(self, leds: list, colors: list, duration: int = 0, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_led_color(self, leds: list, colors: list, duration: int = 0, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_led_color, leds, colors, duration,
-                              callback=callback, event='LedColorDone', sync=sync, chain=chain)
+                              callback=callback, event='LedColorDone', sync=sync, load=load)
     
     def start_led_animation(self, led_group: str, anim_type: str, colors: list, speed: int,
-                            real_blink=False, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+                            real_blink=False, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_led_animation, led_group, anim_type, colors, speed, real_blink,
-                              callback=callback, event='LedAnimationDone', sync=sync, chain=chain)
+                              callback=callback, event='LedAnimationDone', sync=sync, load=load)
 
-    def stop_led_animation(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def stop_led_animation(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).stop_led_animation,
-                              callback=None, event='', sync=sync, chain=chain)
+                              callback=None, event='', sync=sync, load=load)
 
-    def turn(self, degrees: int, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def turn(self, degrees: int, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).turn, degrees,
-                              callback=callback, event='TurnDone', sync=sync, chain=chain)
+                              callback=callback, event='TurnDone', sync=sync, load=load)
 
-    def wake_up(self, callback: callable = None, sync: bool = True, chain: bool = False) -> None:
+    def wake_up(self, callback: callable = None, sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).wake_up,
-                              callback=callback, event='WakeUpDone', sync=sync, chain=chain)
+                              callback=callback, event='WakeUpDone', sync=sync, load=load)
 
-    def rest(self, callback: callable = None, sync: bool = True, chain: bool = False) -> None:
+    def rest(self, callback: callable = None, sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).rest,
-                              callback=callback, event='RestDone', sync=sync, chain=chain)
+                              callback=callback, event='RestDone', sync=sync, load=load)
 
-    def set_breathing(self, enable: bool, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_breathing(self, enable: bool, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_breathing, enable,
-                              callback=callback, event='Breathing' + ('Enabled' if enable else 'Disabled'), sync=sync, chain=chain)
+                              callback=callback, event='Breathing' + ('Enabled' if enable else 'Disabled'), sync=sync, load=load)
 
-    def go_to_posture(self, posture: Enum, speed: int = 100, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
-        """
-        The robot will try for 3 times to reach a position.
-        go_to_posture's callback returns a bool indicating whether the given posture was successfully reached.
-        """
+    def go_to_posture(self, posture: Enum, speed: int = 100, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).go_to_posture, posture.value, speed,
                               callback=partial(self.__posture_callback,
                                                target_posture=posture,
                                                embedded_callback=callback),
-                              event='GoToPostureDone', sync=sync, chain=chain)
+                              event='GoToPostureDone', sync=sync, load=load)
 
     def __posture_callback(self, target_posture: str, embedded_callback: callable) -> None:
         if self.robot_state['posture'] == target_posture:  # if posture was successfully reached
@@ -544,25 +550,25 @@ class BasicSICConnector(AbstractSICConnector):
         else:  # if the posture was not reached
             embedded_callback(False)  # call the listener to signal a failure
 
-    def set_stiffness(self, joints: list, stiffness: int, duration: int = 1000, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def set_stiffness(self, joints: list, stiffness: int, duration: int = 1000, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).set_stiffness, joints, stiffness, duration,
-                              callback=callback, event='SetStiffnessDone', sync=sync, chain=chain)
+                              callback=callback, event='SetStiffnessDone', sync=sync, load=load)
 
-    def play_motion(self, motion: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def play_motion(self, motion: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).play_motion, motion,
-                              callback=callback, event='PlayMotionDone', sync=sync, chain=chain)
+                              callback=callback, event='PlayMotionDone', sync=sync, load=load)
 
-    def start_record_motion(self, joint_chains: list, framerate: int = 5, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def start_record_motion(self, joint_chains: list, framerate: int = 5, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_record_motion, joint_chains, framerate,
-                              callback=callback, event='RecordMotionStarted', sync=sync, chain=chain)
+                              callback=callback, event='RecordMotionStarted', sync=sync, load=load)
 
-    def stop_record_motion(self, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def stop_record_motion(self, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).stop_record_motion,
-                              callback=callback, event='onRobotMotionRecording', sync=sync, chain=chain)
+                              callback=callback, event='onRobotMotionRecording', sync=sync, load=load)
 
-    def browser_show(self, html: str, callback: callable = None,  sync: bool = True, chain: bool = False) -> None:
+    def browser_show(self, html: str, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).browser_show, html,
-                              callback=None, event='', sync=sync, chain=chain)
+                              callback=None, event='', sync=sync, load=load)
 
     ###########################
     # Listeners Management  #
@@ -629,13 +635,6 @@ class BasicSICConnector(AbstractSICConnector):
 
     @staticmethod
     def __build_sync_callback(additional_callback: callable = None):
-        """
-        Builds a callback function that calls set() on an threading.Event() lock when trigger.
-        This is the main mechanism behind a waiting Action.
-
-        :param additional_callback: callback function to be embedded in the waiting callback
-        :return: Callable callback, Event() lock
-        """
         lock = Event()
 
         def callback(*args):
@@ -645,7 +644,7 @@ class BasicSICConnector(AbstractSICConnector):
 
         return callback, lock
 
-    def __process_action(self, action: callable, *args, callback: callable, event: str, sync: bool, chain: bool):
+    def __process_action(self, action: callable, *args, callback: callable, event: str, sync: bool, load: bool):
         inner_callback = callback
         lock = None
 
@@ -655,7 +654,7 @@ class BasicSICConnector(AbstractSICConnector):
         if inner_callback:
             self.__register_action_listener(event, inner_callback)
 
-        if chain:
+        if load:
             self.__loaded_actions.append(Action(action, *args, lock=lock))
         else:
             action(*args)
@@ -663,6 +662,12 @@ class BasicSICConnector(AbstractSICConnector):
                 lock.wait()
 
     def run_loaded_actions(self, wait_for_any=False):
+        """
+        Runs all the loaded actions and activates all loaded listeners in parallel.
+
+        :param wait_for_any: when true locks the main thread until any one of the 'synced' actions are completed.
+        When false waits for *all* of the 'synced' actions to complete.
+        """
         locks = []
         for action in self.__loaded_actions:
             lock = action.perform()
